/*
 * Instructions1.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppnen <--jseppanen@gmail.com-->.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/BasicBlock.h>
#include <llvm/Constant.h>
#include <llvm/Constants.h>
#include <llvm/DerivedTypes.h>
#include <llvm/Function.h>
#include <llvm/Instructions.h>
#include <llvm/Type.h>
#include <llvm/Value.h>

using namespace boost::python;

// Instruction constructor helpers
#include "inst_init.h"

namespace llvm { namespace Python {

template ShiftInst* InstInitFn5(Instruction::OtherOps, Value*, Value*, const std::string&, Instruction*);
template ShiftInst* InstInitFn5(Instruction::OtherOps, Value*, Value*, const std::string&, BasicBlock*);

void wrap_ShiftInst()
{
    // Name C++ overloads
    bool (*classof_ShiftInst)(const ShiftInst*) = &ShiftInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &ShiftInst::classof;
    bool (*classof_Value)(const Value*) = &ShiftInst::classof;

    class_<ShiftInst, boost::noncopyable, bases<Instruction> >
        ("ShiftInst", init<Instruction::OtherOps, Value*, Value*,
                           optional<const std::string> >())
        .def("init", INST_INIT_DEF5(ShiftInst, Instruction::OtherOps, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(ShiftInst, Instruction::OtherOps, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getOpcode", &ShiftInst::getOpcode)
        .def("clone", &ShiftInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_ShiftInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template SelectInst* InstInitFn5(Value*, Value*, Value*, const std::string&, Instruction*);
template SelectInst* InstInitFn5(Value*, Value*, Value*, const std::string&, BasicBlock*);

void wrap_SelectInst()
{
    // Name C++ overloads
    bool (*classof_SelectInst)(const SelectInst*) = &SelectInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &SelectInst::classof;
    bool (*classof_Value)(const Value*) = &SelectInst::classof;

    class_<SelectInst, boost::noncopyable, bases<Instruction> >
        ("SelectInst", init<Value*, Value*, Value*,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF5(SelectInst, Value*, Value*, Value*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF5(SelectInst, Value*, Value*, Value*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getCondition", &SelectInst::getCondition,
             return_internal_reference<>())
        .def("getTrueValue", &SelectInst::getTrueValue,
             return_internal_reference<>())
        .def("getFalseValue", &SelectInst::getFalseValue,
             return_internal_reference<>())
        .def("getOpcode", &SelectInst::getOpcode)
        .def("clone", &SelectInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_SelectInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template VANextInst* InstInitFn4(Value*, const Type*, const std::string&, Instruction*);
template VANextInst* InstInitFn4(Value*, const Type*, const std::string&, BasicBlock*);

void wrap_VANextInst()
{
    // Name C++ overloads
    bool (*classof_VANextInst)(const VANextInst*) = &VANextInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &VANextInst::classof;
    bool (*classof_Value)(const Value*) = &VANextInst::classof;

    class_<VANextInst, boost::noncopyable, bases<Instruction> >
        ("VANextInst", init<Value*, const Type*,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF4(VANextInst, Value*, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(VANextInst, Value*, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("getArgType", &VANextInst::getArgType,
             return_internal_reference<>())
        .def("clone", &VANextInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_VANextInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template VAArgInst* InstInitFn4(Value*, const Type*, const std::string&, Instruction*);
template VAArgInst* InstInitFn4(Value*, const Type*, const std::string&, BasicBlock*);

void wrap_VAArgInst()
{
    // Name C++ overloads
    bool (*classof_VAArgInst)(const VAArgInst*) = &VAArgInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &VAArgInst::classof;
    bool (*classof_Value)(const Value*) = &VAArgInst::classof;

    class_<VAArgInst, boost::noncopyable, bases<Instruction> >
        ("VAArgInst", init<Value*, const Type*,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF4(VAArgInst, Value*, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF4(VAArgInst, Value*, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &VAArgInst::clone,
             return_value_policy<manage_new_object>())
        .def("classof", classof_VAArgInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template PHINode* InstInitFn3(const Type*, const std::string&, Instruction*);
template PHINode* InstInitFn3(const Type*, const std::string&, BasicBlock*);

void wrap_PHINode()
{
    // Name C++ overloads
    Value* (PHINode::*removeIncomingValue)(unsigned, bool) = &PHINode::removeIncomingValue;
    Value* (PHINode::*removeIncomingValue2)(const BasicBlock*, bool) = &PHINode::removeIncomingValue;
    bool (*classof_PHINode)(const PHINode*) = &PHINode::classof;
    bool (*classof_Instruction)(const Instruction*) = &PHINode::classof;
    bool (*classof_Value)(const Value*) = &PHINode::classof;

    class_<PHINode, boost::noncopyable, bases<Instruction> >
        ("PHINode", init<const Type*,
                         optional<const std::string> >())
        .def("init", INST_INIT_DEF3(PHINode, const Type*, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF3(PHINode, const Type*, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &PHINode::clone,
             return_value_policy<manage_new_object>())
        .def("getNumIncomingValues", &PHINode::getNumIncomingValues)
        .def("getIncomingValue", &PHINode::getIncomingValue,
             return_internal_reference<>())
        .def("setIncomingValue", &PHINode::setIncomingValue)
        .def("getOperandNumForIncomingValue", &PHINode::getOperandNumForIncomingValue)
        .def("getIncomingBlock", &PHINode::getIncomingBlock,
             return_internal_reference<>())
        .def("setIncomingBlock", &PHINode::setIncomingBlock)
        .def("getOperandNumForIncomingBlock", &PHINode::getOperandNumForIncomingBlock)
        .def("addIncoming", &PHINode::addIncoming)
        .def("removeIncomingValue", removeIncomingValue,
             return_value_policy<manage_new_object>())
        .def("removeIncomingValue", removeIncomingValue2,
             return_value_policy<manage_new_object>())
        .def("getBasicBlockIndex", &PHINode::getBasicBlockIndex)
        .def("getIncomingValueForBlock", &PHINode::getIncomingValueForBlock,
             return_internal_reference<>())
        .def("classof", classof_PHINode)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template ReturnInst* InstInitFn2(Value *RetVal, Instruction *InsertBefore);
template ReturnInst* InstInitFn2(Value *RetVal, BasicBlock *InsertAtEnd);
template ReturnInst* InstInitFn1(BasicBlock *InsertAtEnd);

void wrap_ReturnInst()
{
    // Name C++ overloads
    bool (*classof_ReturnInst)(const ReturnInst*) = &ReturnInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &ReturnInst::classof;
    bool (*classof_Value)(const Value*) = &ReturnInst::classof;
    
    class_<ReturnInst, std::auto_ptr<ReturnInst>, boost::noncopyable, bases<TerminatorInst> >
        ("ReturnInst", init<optional<Value*> >())
        .def("init", INST_INIT_DEF2(ReturnInst, Value*, Instruction*))
        .def("init", INST_INIT_DEF2(ReturnInst, Value*, BasicBlock*))
        .def("init", INST_INIT_DEF1(ReturnInst, BasicBlock*))
        .staticmethod("init")

        .def("clone", &ReturnInst::clone,
             return_value_policy<manage_new_object>())
        .def("getReturnValue", &ReturnInst::getReturnValue,
             return_internal_reference<>())
        .def("getSuccessor", &ReturnInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &ReturnInst::setSuccessor)
        .def("getNumSuccessors", &ReturnInst::getNumSuccessors)
        .def("classof", classof_ReturnInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template BranchInst* InstInitFn2(BasicBlock*, Instruction*);
template BranchInst* InstInitFn4(BasicBlock*, BasicBlock*, Value*, Instruction*);
template BranchInst* InstInitFn2(BasicBlock*, BasicBlock*);
template BranchInst* InstInitFn4(BasicBlock*, BasicBlock*, Value*, BasicBlock*);

void wrap_BranchInst()
{
    // Name C++ overloads
    bool (*classof_BranchInst)(const BranchInst*) = &BranchInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &BranchInst::classof;
    bool (*classof_Value)(const Value*) = &BranchInst::classof;

    class_<BranchInst, boost::noncopyable, bases<TerminatorInst> >
        ("BranchInst", init<BasicBlock*>())
        .def(init<BasicBlock*, BasicBlock*, Value*>())
        .def("init", INST_INIT_DEF2(BranchInst, BasicBlock*, Instruction*))
        .def("init", INST_INIT_DEF4(BranchInst, BasicBlock*, BasicBlock*, Value*, Instruction*))
        .def("init", INST_INIT_DEF2(BranchInst, BasicBlock*, BasicBlock*))
        .def("init", INST_INIT_DEF4(BranchInst, BasicBlock*, BasicBlock*, Value*, BasicBlock*))
        .staticmethod("init")

        .def("clone", &BranchInst::clone,
             return_value_policy<manage_new_object>())
        .def("isUnconditional", &BranchInst::isUnconditional)
        .def("isConditional", &BranchInst::isConditional)
        .def("getCondition", &BranchInst::getCondition,
             return_internal_reference<>())
        .def("setCondition", &BranchInst::setCondition)
        .def("setUnconditionalDest", &BranchInst::setUnconditionalDest)
        .def("getSuccessor", &BranchInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &BranchInst::setSuccessor)
        .def("getNumSuccessors", &BranchInst::getNumSuccessors)
        .def("classof", classof_BranchInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template SwitchInst* InstInitFn4(Value*, BasicBlock*, unsigned, Instruction*);
template SwitchInst* InstInitFn4(Value*, BasicBlock*, unsigned, BasicBlock *);

void wrap_SwitchInst()
{
    // Name C++ overloads
    const ConstantInt* (SwitchInst::*getCaseValue)(unsigned) const = &SwitchInst::getCaseValue;
    bool (*classof_SwitchInst)(const SwitchInst*) = &SwitchInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &SwitchInst::classof;
    bool (*classof_Value)(const Value*) = &SwitchInst::classof;

    class_<SwitchInst, boost::noncopyable, bases<TerminatorInst> >
        ("SwitchInst", init<Value*, BasicBlock*, unsigned>())
        .def("init", INST_INIT_DEF4(SwitchInst, Value*, BasicBlock*, unsigned, Instruction*))
        .def("init", INST_INIT_DEF4(SwitchInst, Value*, BasicBlock*, unsigned, BasicBlock *))
        .staticmethod("init")

        .def("clone", &SwitchInst::clone,
             return_value_policy<manage_new_object>())
        .def("getCondition", &SwitchInst::getCondition,
             return_internal_reference<>())
        .def("getDefaultDest", &SwitchInst::getDefaultDest,
             return_internal_reference<>())
        .def("getNumCases", &SwitchInst::getNumCases)
        .def("getCaseValue", getCaseValue,
             return_internal_reference<>())
        .def("findCaseValue", &SwitchInst::findCaseValue)
        .def("addCase", &SwitchInst::addCase)
        .def("removeCase", &SwitchInst::removeCase)
        .def("getSuccessor", &SwitchInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &SwitchInst::setSuccessor)
        .def("getSuccessorValue", &SwitchInst::getSuccessorValue,
             return_internal_reference<>())
        .def("getNumSuccessors", &SwitchInst::getNumSuccessors)
        .def("classof", classof_SwitchInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template InvokeInst* InstInitFn6(Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, Instruction*);
template InvokeInst* InstInitFn6(Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, BasicBlock*);

void wrap_InvokeInst()
{
    // Name C++ overloads
    bool (*classof_InvokeInst)(const InvokeInst*) = &InvokeInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &InvokeInst::classof;
    bool (*classof_Value)(const Value*) = &InvokeInst::classof;

    class_<InvokeInst, boost::noncopyable, bases<TerminatorInst> >
        ("InvokeInst", init<Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&,
                            optional<const std::string> >())
        .def("init", INST_INIT_DEF6(InvokeInst, Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, Instruction*))
        .def("init", INST_INIT_DEF6(InvokeInst, Value*, BasicBlock*, BasicBlock*, const std::vector<Value*>&, const std::string&, BasicBlock*))
        .staticmethod("init")

        .def("clone", &InvokeInst::clone,
             return_value_policy<manage_new_object>())
        .def("mayWriteToMemory", &InvokeInst::mayWriteToMemory)
        .def("getCalledFunction", &InvokeInst::getCalledFunction,
             return_internal_reference<>())
        .def("getCalledValue", &InvokeInst::getCalledValue,
             return_internal_reference<>())
        .def("getNormalDest", &InvokeInst::getNormalDest,
             return_internal_reference<>())
        .def("getUnwindDest", &InvokeInst::getUnwindDest,
             return_internal_reference<>())
        .def("setNormalDest", &InvokeInst::setNormalDest)
        .def("setUnwindDest", &InvokeInst::setUnwindDest)
        .def("getSuccessor", &InvokeInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &InvokeInst::setSuccessor)
        .def("getNumSuccessors", &InvokeInst::getNumSuccessors)
        .def("classof", classof_InvokeInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template UnwindInst* InstInitFn1(Instruction*);
template UnwindInst* InstInitFn1(BasicBlock*);

void wrap_UnwindInst()
{
    // Name C++ overloads
    bool (*classof_UnwindInst)(const UnwindInst*) = &UnwindInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &UnwindInst::classof;
    bool (*classof_Value)(const Value*) = &UnwindInst::classof;

    class_<UnwindInst, boost::noncopyable, bases<TerminatorInst> >
        ("UnwindInst")
        .def("init", INST_INIT_DEF1(UnwindInst, Instruction*))
        .def("init", INST_INIT_DEF1(UnwindInst, BasicBlock*))
        .staticmethod("init")

        .def("clone", &UnwindInst::clone,
             return_value_policy<manage_new_object>())
        .def("getSuccessor", &UnwindInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &UnwindInst::setSuccessor)
        .def("getNumSuccessors", &UnwindInst::getNumSuccessors)
        .def("classof", classof_UnwindInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

template UnreachableInst* InstInitFn1(Instruction*);
template UnreachableInst* InstInitFn1(BasicBlock*);

void wrap_UnreachableInst()
{
    // Name C++ overloads
    bool (*classof_UnreachableInst)(const UnreachableInst*) = &UnreachableInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &UnreachableInst::classof;
    bool (*classof_Value)(const Value*) = &UnreachableInst::classof;

    class_<UnreachableInst, boost::noncopyable, bases<TerminatorInst> >
        ("UnreachableInst")
        .def("init", INST_INIT_DEF1(UnreachableInst, Instruction*))
        .def("init", INST_INIT_DEF1(UnreachableInst, BasicBlock*))
        .staticmethod("init")

        .def("clone", &UnreachableInst::clone,
             return_value_policy<manage_new_object>())
        .def("getSuccessor", &UnreachableInst::getSuccessor,
             return_internal_reference<>())
        .def("setSuccessor", &UnreachableInst::setSuccessor)
        .def("getNumSuccessors", &UnreachableInst::getNumSuccessors)
        .def("classof", classof_UnreachableInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_Instructions2()
{
    wrap_ShiftInst();
    wrap_SelectInst();
    wrap_VANextInst();
    wrap_VAArgInst();
    wrap_PHINode();
    wrap_ReturnInst();
    wrap_BranchInst();
    wrap_SwitchInst();
    wrap_InvokeInst();
    wrap_UnwindInst();
    wrap_UnreachableInst();
}

}} // namespace llvm::Python

// EOF
