/*
 * SymbolTable.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppnen <--jseppanen@gmail.com-->.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>

#include "std_pair.h"

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_SymbolTable()
{
    // Name C++ overloads
    void (SymbolTable::*insert1)(Value*) = &SymbolTable::insert;
    void (SymbolTable::*insert2)(const std::string&, const Type*) = &SymbolTable::insert;
    void (SymbolTable::*remove1)(Value*) = &SymbolTable::remove;
    Type* (SymbolTable::*remove2)(SymbolTable::type_iterator) = &SymbolTable::remove;
    SymbolTable::plane_const_iterator (SymbolTable::*plane_begin)() const = &SymbolTable::plane_begin;
    SymbolTable::plane_const_iterator (SymbolTable::*plane_end)() const = &SymbolTable::plane_end;
    SymbolTable::value_const_iterator (SymbolTable::*value_begin)(const Type*) const = &SymbolTable::value_begin;
    SymbolTable::value_const_iterator (SymbolTable::*value_end)(const Type*) const = &SymbolTable::value_end;
    SymbolTable::type_const_iterator (SymbolTable::*type_begin)() const = &SymbolTable::type_begin;
    SymbolTable::type_const_iterator (SymbolTable::*type_end)() const = &SymbolTable::type_end;
    SymbolTable::plane_const_iterator (SymbolTable::*find)(const Type*) const = &SymbolTable::find;

    class_<SymbolTable, boost::noncopyable, bases<AbstractTypeUser> >("SymbolTable")

        .def("lookup", &SymbolTable::lookup,
             return_internal_reference<>())
        .def("lookupType", &SymbolTable::lookupType,
             return_internal_reference<>())
        .def("isEmpty", &SymbolTable::isEmpty)
        .def("num_types", &SymbolTable::num_types)
        .def("getUniqueName", &SymbolTable::getUniqueName)
        .def("dump", &SymbolTable::dump)
        .def("insert", insert1,
             with_custodian_and_ward<1, 2>()) // create Value* dependency
        .def("insert", insert2,
             with_custodian_and_ward<1, 3>()) // create Type* dependency
        //FIXME cannot destroy pointer dependencies?
        .def("remove", remove1)
        // the return values are NOT actually owned by SymbolTable, but
        // this assumes the lifetime dependency created during insert().
        .def("remove", remove2,
             return_internal_reference<>())
        .def("strip", &SymbolTable::strip)
        .add_property("planes", range(plane_begin, plane_end))
        //.def("plane_begin", plane_begin)
        //.def("plane_end", plane_end)
        // FIXME Type* argument cannot be mapped into range
        //.def("values", range(value_begin, value_end))
        .def("value_begin", value_begin)
        .def("value_end", value_end)
        .add_property("types", range(type_begin, type_end))
        //.def("type_begin", type_begin)
        //.def("type_end", type_end)
        .def("find", find)
        .def("changeName", &SymbolTable::changeName)
        ;

    // FIXME plane iterators (translate ValueMap into iterator?)
    std_pair_to_python_converter<const Type*, SymbolTable::ValueMap>();
    // value iterators
    std_pair_to_python_converter<const std::string, const Value*>();
    // type iterators
    std_pair_to_python_converter<const std::string, const Type*>();
}

}} // namespace llvm::Python

// EOF
