/*
 * Transforms_IPO.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppnen <--jseppanen@gmail.com-->.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Transforms/IPO.h>
#include <llvm/BasicBlock.h>
#include <llvm/Function.h>
#include <llvm/Pass.h>

using namespace boost::python;

namespace llvm { namespace Python {

BOOST_PYTHON_FUNCTION_OVERLOADS(createStripSymbolsPass_overloads, createStripSymbolsPass, 0, 1)
BOOST_PYTHON_FUNCTION_OVERLOADS(createFunctionExtractionPass_overloads, createFunctionExtractionPass, 1, 2)

void wrap_Transforms_IPO()
{
    def("createStripSymbolsPass", createStripSymbolsPass,
        createStripSymbolsPass_overloads()
        [return_value_policy<manage_new_object>()]);
    def("createLowerSetJmpPass", createLowerSetJmpPass,
        return_value_policy<manage_new_object>());
    def("createConstantMergePass", createConstantMergePass,
        return_value_policy<manage_new_object>());
    def("createGlobalOptimizerPass", createGlobalOptimizerPass,
        return_value_policy<manage_new_object>());
    def("createRaiseAllocationsPass", createRaiseAllocationsPass,
        return_value_policy<manage_new_object>());
    def("createDeadTypeEliminationPass", createDeadTypeEliminationPass,
        return_value_policy<manage_new_object>());
    def("createGlobalDCEPass", createGlobalDCEPass,
        return_value_policy<manage_new_object>());
    def("createFunctionExtractionPass", createFunctionExtractionPass,
        createFunctionExtractionPass_overloads()
        [return_value_policy<manage_new_object>()]);
    def("createFunctionResolvingPass", createFunctionResolvingPass,
        return_value_policy<manage_new_object>());
    def("createFunctionInliningPass", createFunctionInliningPass,
        return_value_policy<manage_new_object>());
    def("createPruneEHPass", createPruneEHPass,
        return_value_policy<manage_new_object>());
    def("createInternalizePass", createInternalizePass,
        return_value_policy<manage_new_object>());
    def("createDeadArgEliminationPass", createDeadArgEliminationPass,
        return_value_policy<manage_new_object>());
    def("createDeadArgHackingPass", createDeadArgHackingPass,
        return_value_policy<manage_new_object>());
    def("createArgumentPromotionPass", createArgumentPromotionPass,
        return_value_policy<manage_new_object>());
    def("createIPConstantPropagationPass", createIPConstantPropagationPass,
        return_value_policy<manage_new_object>());
    def("createIPSCCPPass", createIPSCCPPass,
        return_value_policy<manage_new_object>());
    def("createLoopExtractorPass", createLoopExtractorPass,
        return_value_policy<manage_new_object>());
    def("createSingleLoopExtractorPass", createSingleLoopExtractorPass,
        return_value_policy<manage_new_object>());
    //FIXME wrap std::vector<BasicBlock*>
    def("createBlockExtractorPass", createBlockExtractorPass,
        return_value_policy<manage_new_object>());
    def("createSimplifyLibCallsPass", createSimplifyLibCallsPass,
        return_value_policy<manage_new_object>());
}

}} // namespace llvm::Python

// EOF
