/*
 * Transforms_Scalar.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppnen <--jseppanen@gmail.com-->.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Transforms/Scalar.h>
#include <llvm/InstrTypes.h>
#include <llvm/Instructions.h>
#include <llvm/Pass.h>

using namespace boost::python;

namespace llvm { namespace Python {

BOOST_PYTHON_FUNCTION_OVERLOADS(createLoopStrengthReducePass_overloads, createLoopStrengthReducePass, 0, 1)
BOOST_PYTHON_FUNCTION_OVERLOADS(createLowerAllocationsPass_overloads, createLowerAllocationsPass, 0, 1)
BOOST_PYTHON_FUNCTION_OVERLOADS(createLowerSelectPass_overloads, createLowerSelectPass, 0, 1)

void wrap_Transforms_Scalar()
{
    def("createRaisePointerReferencesPass", createRaisePointerReferencesPass,
        return_value_policy<manage_new_object>());
    def("createConstantPropagationPass", createConstantPropagationPass,
        return_value_policy<manage_new_object>());
    def("createSCCPPass", createSCCPPass,
        return_value_policy<manage_new_object>());
    def("createDeadInstEliminationPass", createDeadInstEliminationPass,
        return_value_policy<manage_new_object>());
    def("createDeadCodeEliminationPass", createDeadCodeEliminationPass,
        return_value_policy<manage_new_object>());
    def("createDeadStoreEliminationPass", createDeadStoreEliminationPass,
        return_value_policy<manage_new_object>());
    def("createAggressiveDCEPass", createAggressiveDCEPass,
        return_value_policy<manage_new_object>());
    def("createScalarReplAggregatesPass", createScalarReplAggregatesPass,
        return_value_policy<manage_new_object>());
    def("createGCSEPass", createGCSEPass,
        return_value_policy<manage_new_object>());
    def("createIndVarSimplifyPass", createIndVarSimplifyPass,
        return_value_policy<manage_new_object>());
    def("createInstructionCombiningPass", createInstructionCombiningPass,
        return_value_policy<manage_new_object>());
    def("createLICMPass", createLICMPass,
        return_value_policy<manage_new_object>());
    def("createLoopStrengthReducePass", createLoopStrengthReducePass,
        createLoopStrengthReducePass_overloads()
        [return_value_policy<manage_new_object>()]);
    def("createLoopUnswitchPass", createLoopUnswitchPass,
        return_value_policy<manage_new_object>());
    def("createLoopUnrollPass", createLoopUnrollPass,
        return_value_policy<manage_new_object>());
    def("createPromoteMemoryToRegisterPass", createPromoteMemoryToRegisterPass,
        return_value_policy<manage_new_object>());
    def("createReassociatePass", createReassociatePass,
        return_value_policy<manage_new_object>());
    def("createCorrelatedExpressionEliminationPass", createCorrelatedExpressionEliminationPass,
        return_value_policy<manage_new_object>());
    def("createCondPropagationPass", createCondPropagationPass,
        return_value_policy<manage_new_object>());
    def("createTailDuplicationPass", createTailDuplicationPass,
        return_value_policy<manage_new_object>());
    def("createCFGSimplificationPass", createCFGSimplificationPass,
        return_value_policy<manage_new_object>());
    def("createBreakCriticalEdgesPass", createBreakCriticalEdgesPass,
        return_value_policy<manage_new_object>());
    //extern const PassInfo *BreakCriticalEdgesID;
    def("createLoopSimplifyPass", createLoopSimplifyPass,
        return_value_policy<manage_new_object>());
    //extern const PassInfo *LoopSimplifyID;
    def("createTailCallEliminationPass", createTailCallEliminationPass,
        return_value_policy<manage_new_object>());
    def("createLowerAllocationsPass", createLowerAllocationsPass,
        createLowerAllocationsPass_overloads()
        [return_value_policy<manage_new_object>()]);
    def("createLowerSwitchPass", createLowerSwitchPass,
        return_value_policy<manage_new_object>());
    def("createLowerSelectPass", createLowerSelectPass,
        createLowerSelectPass_overloads()
        [return_value_policy<manage_new_object>()]);
    def("createLowerPackedPass", createLowerPackedPass,
        return_value_policy<manage_new_object>());
    def("createLowerInvokePass", createLowerInvokePass,
        return_value_policy<manage_new_object>());
    //extern const PassInfo *LowerInvokePassID;
    def("createLowerGCPass", createLowerGCPass,
        return_value_policy<manage_new_object>());
    def("createLowerConstantExpressionsPass", createLowerConstantExpressionsPass,
        return_value_policy<manage_new_object>());
    def("createBlockPlacementPass", createBlockPlacementPass,
        return_value_policy<manage_new_object>());
    def("createPREPass", createPREPass,
        return_value_policy<manage_new_object>());
}

}} // namespace llvm::Python

// EOF
