/*
 * llvm.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppnen <--jseppanen@gmail.com-->.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <iostream>
#include <llvm/Config/config.h> // PACKAGE_VERSION etc.

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_AbstractTypeUser();
void wrap_Analysis_Verifier();
void wrap_Annotation();
void wrap_Argument();
void wrap_BasicBlock();
void wrap_Bytecode_Reader();
void wrap_Bytecode_Writer();
void wrap_CallGraphSCCPass();
void wrap_Constant();
void wrap_Constants();
void wrap_DerivedTypes();
void wrap_ExecutionEngine();
void wrap_Function();
void wrap_GenericValue();
void wrap_GlobalValue();
void wrap_GlobalVariable();
void wrap_InstrTypes();
void wrap_Instruction();
void wrap_Instructions1();
void wrap_Instructions2();
void wrap_IntrinsicInst();
void wrap_Intrinsics(object module);
void wrap_Linker();
void wrap_Module();
void wrap_ModuleProvider();
void wrap_Pass();
void wrap_PassAnalysisSupport();
void wrap_PassManager();
void wrap_PassSupport();
void wrap_SymbolTable();
void wrap_Type();
void wrap_Use();
void wrap_User();
void wrap_Value();
void wrap_System_Path(object module);
void wrap_Transforms_IPO();
void wrap_Transforms_Instrumentation();
void wrap_Transforms_Scalar();

#if !defined(STRINGIFY) && !defined(EXPAND_AND_STRINGIFY)
# define STRINGIFY(x) #x
# define EXPAND_AND_STRINGIFY(x) STRINGIFY(x)
#endif

BOOST_PYTHON_MODULE(llvm)
{
    // Module identification
    scope().attr("__author__") = "Jarno Seppnen <--jseppanen@gmail.com-->";
    scope().attr("__version__") = "0.2 (" PACKAGE_NAME " " EXPAND_AND_STRINGIFY(PACKAGE_VERSION) ")";
    scope().attr("__copyright__") = "Python-LLVM bindings Copyright (c) 2004, 2005 Jarno Seppnen.  LLVM release Copyright (c) 2003, 2004 University of Illinois at Urbana-Champaign.";
    scope().attr("__license__") = "Python-LLVM bindings distributed under the Boost Software License, Version 1.0 (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt).  LLVM release distributed under the University of Illinois/NCSA Open Source License (see http://llvm.cs.uiuc.edu).";
    scope().attr("__date__") = __DATE__ ", " __TIME__; // build date

    // Create sub-namespaces
    // This doesn't compile with GCC 3.3:
    //object IntrinsicModule(handle<>(PyModule_New((char*)"llvm.Intrinsic")));
    //object sysModule(handle<>(PyModule_New((char*)"llvm.sys")));
    handle<> IntrinsicModuleHandle(PyModule_New((char*)"llvm.Intrinsic"));
    handle<> sysModuleHandle(PyModule_New((char*)"llvm.sys"));
    object IntrinsicModule(IntrinsicModuleHandle);
    object sysModule(sysModuleHandle);
    scope().attr("Intrinsic") = IntrinsicModule;
    scope().attr("sys") = sysModule;

    // wrapping order is significant (bases first)

    // VMCore
    wrap_Value();
    wrap_Type();
    wrap_User();
    wrap_AbstractTypeUser();
    wrap_BasicBlock();
    wrap_Constant();
    wrap_Constants();
    wrap_DerivedTypes();
    wrap_Module();
    wrap_Linker();
    wrap_ModuleProvider();
    wrap_Use();
    wrap_Argument();
    wrap_Annotation();
    wrap_GlobalValue();
    wrap_GlobalVariable();
    wrap_Function();
    wrap_Instruction();
    wrap_InstrTypes();
    wrap_Instructions1();
    wrap_Instructions2();
    wrap_IntrinsicInst();
    wrap_Intrinsics(IntrinsicModule);
    wrap_GenericValue();
    wrap_ExecutionEngine();
    wrap_Pass();
    wrap_PassAnalysisSupport();
    wrap_PassManager();
    wrap_PassSupport();
    wrap_CallGraphSCCPass();
    wrap_SymbolTable();

    // Analysis
    wrap_Analysis_Verifier();

    // Bytecode
    wrap_Bytecode_Reader();
    wrap_Bytecode_Writer();

    // System
    wrap_System_Path(sysModule);

    // Transforms
    wrap_Transforms_IPO();
    wrap_Transforms_Instrumentation();
    wrap_Transforms_Scalar();
}

}} // namespace llvm::Python

// EOF
