"""
Test instruction ownership.

'Stand-alone' instructions are constructed normally.

>>> retval=llvm.ConstantSInt.get(llvm.Type.IntTy,42)
>>> ret=llvm.ReturnInst(retval)
>>> ret.dump() # prints 'ret int 42' to C stdout

Instructions can be constructed directly into BasicBlocks.  Please
note that you cannot use normal constructor in that case but have to
use the init() factory method.  This is because the ownership of the
constructed instruction goes directly to the basic block.  It is
currently not possible to build regular __init__ methods with such
semantics using Boost Python.

>>> b=llvm.BasicBlock()
>>> b.size()
0
>>> r=llvm.ReturnInst.init(b)
>>> b.dump() # prints 'ret void' to C stdout
>>> b.size()
1
>>> del r
>>> gc.collect()
0
>>> b.size()
1

BinaryOperator.create* methods are very useful.

>>> x=llvm.ConstantSInt.get(llvm.Type.IntTy, 1)
>>> y=llvm.ConstantSInt.get(llvm.Type.IntTy, 2)
>>> add=llvm.BinaryOperator.createAdd(x, y, "")

You can also work backwards from an instruction, inspecting its
operands.  The C++ iterators are represented as Python iterators.

>>> for op in add.operands:
...   print op.get().getValue()
...
1
2

"""

import doctest, gc, llvm

if __name__=="__main__":
    doctest.testmod()
