"""
Trying out the JIT execution engine.

Load hello world program from bytecode, get its main() function, and
run it with the JIT engine.  The program prints 'hello' to C stdout
(not Python stdout) and returns zero.

>>> mp=llvm.getBytecodeModuleProvider('hello.ll.bc')
>>> main=mp.getModule().getMainFunction()
>>> jit=llvm.ExecutionEngine.create(mp, False)
>>> ret=jit.runFunction(main, ()) # prints hello
>>> ret.IntVal
0

The ExecutionEngine.create(mp, False) method call assumes ownership of
its first argument, the ModuleProvider.  That is reflected in Python
so that the mp argument becomes 'zombie' and is no longer usable.
This is done on purpose to prevent using a dangling pointer.  However,
the zombie objects can be hard to recognize because they appear as to
be OK until attribute access is attempted.  The error messages in this
case are not very descriptive either.

>>> mp
<llvm.ModuleProvider object at 0x00A2CD88>
>>> help(mp.getModule)
Help on method getModule:
<BLANKLINE>
getModule(...) method of llvm.ModuleProvider instance
<BLANKLINE>
>>> mp.getModule()
Traceback (most recent call last):
  File "c:\Python24\lib\doctest.py", line 1243, in __run
    compileflags, 1) in test.globs
  File "<doctest __main__[7]>", line 1, in ?
    mp.getModule()
ArgumentError: Python argument types in
    ModuleProvider.getModule(ModuleProvider)
did not match C++ signature:
    getModule(class llvm::ModuleProvider {lvalue})

There is nothing much to do about zombie objects than to delete.  You
can find from the LLVM API documentation which of the API calls
implement transfer-of-ownership semantics for their arguments.

>>> del mp

"""

import doctest, llvm

if __name__=="__main__":
    doctest.testmod()
